# Generated by roxygen2: do not edit by hand

export(LogBip)
export(cv_LogBip)
export(fitted_LB)
export(gradientDesc)
export(performanceBLB)
export(plotBLB)
export(pred_LB)
export(proj_LogBip)
export(sdv_MM)
export(simBin)
importFrom(optimx,optimr)
importFrom(optimx,optimx)
importFrom(stats,IQR)
importFrom(stats,plogis)
importFrom(stats,qlogis)
importFrom(stats,rbinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
