utils::globalVariables(c("rowId", "Dim1", "Dimb1", "Dim2", "Dimb2", "ind", "resample", "Dim1c", "Dim2c",
                         "b0", "bb0", "b1", "bb1", "b2", "bb2", "bj0c", "bj1c", "bj2c",
                         "resampleb", "x.50", "x.75", "y.50", "y.75",
                         "threshold", "variable", "cv-error", "merror",
                         "thresholds", "error", "type.error", "nclass", "sel"))
